@@ -3,13 +3,13 @@ package Crypt::Passphrase::Pepper::HSM;
 use strict;
 use warnings;
 
-our $VERSION = '0.003';
+our $VERSION = '0.004';
 
 use parent 'Crypt::Passphrase::Pepper::Base';
-use Crypt::Passphrase -encoder;
+use Crypt::Passphrase 0.016 -encoder;
 
 use Carp 'croak';
-use Crypt::HSM;
+use Crypt::HSM 0.010;
 
 sub new {
 	my ($class, %args) = @_;
@@ -18,9 +18,9 @@ sub new {
 	$args{prefix} //= 'pepper-';
 
 	$args{session} //= do {
-		my $provider = ref $args{provider} ? $args{provider} : Crypt::HSM->load(delete $args{provider});
+		my $provider = ref $args{provider} ? delete $args{provider} : Crypt::HSM->load(delete $args{provider});
 		my $slot = delete $args{slot} // ($provider->slots)[0];
-		$provider->open_session($slot);
+		$slot->open_session;
 	};
 	my $user_type = delete $args{user_type} // 'user';
 	$args{session}->login($user_type, delete $args{pin}) if $args{pin};
@@ -28,6 +28,11 @@ sub new {
 	return $class->SUPER::new(%args);
 }
 
+sub supported_hashes {
+	my ($self) = @_;
+	return map { $_->name } grep { $_->has_flags('sign') && $_->min_key_size <= 64 } $self->{session}->slot->mechanisms;
+}
+
 sub prehash_password {
 	my ($self, $password, $algorithm, $id) = @_;
 
